import net from 'net'
import http from 'http'
import logger from './logger.js'
import { getDevServerURL } from './middlewares/detect-dev-server.js'

export function createHttpProxy ({ target, port = 8080 } = {}) {
  const server = http.createServer().listen(port, () => {
    logger.info(`Created http proxy on port ${port}`)
  })

  server.on('connect', (req, clientSocket, head) => {
    let serverUrl
    const reqHost = req.headers.host.split(':')[0]
    const reqPort = req.headers.host.split(':')[1] || '443'
    // default to 443, that's what browsers do
    const serverPort = (target.match(/:([0-9]+)\//) || [])[1] || '443'
    const devServerUrl = getDevServerURL()
    if (!devServerUrl) return logger.info('Receveid connect event but dev server is not ready yet')
    if (target.indexOf(reqHost) >= 0) {
      // intended appserver connection
      serverUrl = {}
      serverUrl.hostname = devServerUrl.hostname
      if (reqPort !== serverPort) {
        // do not change the port if not meant for the dev server
        serverUrl.port = reqPort
      } else {
        serverUrl.port = devServerUrl.port
      }
    } else {
      serverUrl = new URL(`https://${req.url}`)
    }
    logger.trace(`http proxy: proxy connect from "${reqHost}:${reqPort}" to "${serverUrl.hostname}.${serverUrl.port || '443'}"`)
    const serverSocket = net.connect(serverUrl.port || '443', serverUrl.hostname, function () {
      clientSocket.write('HTTP/1.1 200 Connection Established\r\n' +
        'Proxy-agent: Node-Proxy\r\n' +
        '\r\n')
      serverSocket.write(head)
      // reset timeout to default, because the connection now most likely works
      serverSocket.setTimeout(120000)
      serverSocket.pipe(clientSocket)
      clientSocket.pipe(serverSocket)
    })
    // add a quite low timeout value, because the proxy will always be reachable, but the remote might not
    serverSocket.setTimeout(10000)
    serverSocket.on('timeout', function () {
      serverSocket.end()
      clientSocket.end()
    })

    clientSocket.on('error', function (err) {
      logger.error(`Http proxy client socket error: ${err.stack}`)
    })
    serverSocket.on('error', function (err) {
      logger.error(`Http proxy server socket error: ${err.stack}`)
    })
  })
  server.on('clientError', function (err) {
    logger.error(`Http proxy clientError: ${err.stack}`)
  })
}
