import { describe, it, afterEach } from '@jest/globals'
import path from 'path'
import vitePluginProxy from '../../src/index'
import { closeAllServers, createViteServer, getPort, http2Request, updateRequestManager } from '../util'

const __dirname = path.dirname(new URL(import.meta.url).pathname)
const PORT = getPort()

describe('detect dev server', function () {
  afterEach(async function () {
    await closeAllServers()
    updateRequestManager()
  })

  it('works', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy()]
    }, PORT)

    // need to launch a request to trigger own server detection
    await http2Request(`http://localhost:${PORT}/index.js`)
  })

  it('works with all hosts config', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy()],
      server: {
        host: '0.0.0.0'
      }
    }, PORT)

    // need to launch a request to trigger own server detection
    await http2Request(`http://0.0.0.0:${PORT}/index.js`)
  })

  it('works with all hosts config ipv6', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy()],
      server: {
        host: '::'
      }
    }, PORT)

    // need to launch a request to trigger own server detection
    await http2Request(`http://localhost:${PORT}/index.js`)
  })
})
